| fastbeta-deprecated | R Documentation |
The functions and other objects listed here are provided only for compatibility with older versions of fastbeta and may become defunct as soon as the next release.
## Nothing yet!
Defunct, base-defunct,
fastbeta-defunct, fastbeta-notyet.
Add the following code to your website.
For more information on customizing the embed code, read Embedding Snippets.