Calculate the basic reproduction number, endemic equilibrium, and Jacobian matrix of the SEIR model without forcing.
seir.R0 (beta, nu = 0, mu = 0, sigma = 1, gamma = 1, delta = 0,
m = 1L, n = 1L, N = 1)
seir.ee (beta, nu = 0, mu = 0, sigma = 1, gamma = 1, delta = 0,
m = 1L, n = 1L, N = 1)
seir.jacobian(beta, nu = 0, mu = 0, sigma = 1, gamma = 1, delta = 0,
m = 1L, n = 1L)seir.R0 returns a numeric vector of length 1. seir.ee
returns a numeric vector of length 1+m+n+1. seir.jacobian
returns a function of one argument x (which must be a numeric
vector of length 1+m+n+1) whose return value is a square numeric
matrix of size length(x).
non-negative numbers. beta, nu, and mu are the
rates of transmission, birth, and natural death. m*sigma,
n*gamma, and delta are the rates of removal from each
latent, infectious, and recovered compartment.
a non-negative integer indicating a number of latent stages.
a positive integer indicating a number of infectious stages.
a non-negative number indicating a population size for the
(nu == 0 && mu == 0) case.
If \(\mu, \nu = 0\), then the basic reproduction number is computed as $$ \mathcal{R}_{0} = N \beta / \gamma $$ and the endemic equilibrium is computed as $$ \begin{bmatrix} S^{\hphantom{1}} \\ E^{i} \\ I^{j} \\ R^{\hphantom{1}} \end{bmatrix} = \begin{bmatrix} \gamma / \beta \\ w \delta / (m \sigma) \\ w \delta / (n \gamma) \\ w \end{bmatrix} $$ where \(w\) is chosen so that the sum is \(N\).
If \(\mu, \nu > 0\), then the basic reproduction number is computed as $$ \mathcal{R}_{0} = \nu \beta a^{-m} (1 - b^{-n}) / \mu^{2} $$ and the endemic equilibrium is computed as $$ \begin{bmatrix} S^{\hphantom{1}} \\ E^{i} \\ I^{j} \\ R^{\hphantom{1}} \end{bmatrix} = \begin{bmatrix} \mu a^{m} / (\beta (1 - b^{-n})) \\ w a^{m - i} b^{n} (\delta + \mu) / (m \sigma) \\ w b^{n - j} (\delta + \mu) / (n \gamma) \\ w \end{bmatrix} $$ where \(w\) is chosen so that the sum is \(\nu / \mu\), the population size at equilibrium, and \(a = 1 + \mu / (m \sigma)\) and \(b = 1 + \mu / (n \gamma)\).
Currently, none of the functions documented here are vectorized. Arguments must have length 1.
seir, for the system of ordinary differential equations
on which these computations are predicated.